import 'package:flutter/services.dart';

class BrazePlugin {
  static const MethodChannel _channel = const MethodChannel('braze_plugin');

  /// Changes the current Braze userId
  void changeUser(String userId) {
    final Map<String, dynamic> params = <String, dynamic>{"userId": userId};
    _channel.invokeMethod('changeUser', params);
  }

  /// Logs a custom event to Braze
  void logCustomEvent(String eventName) {
    final Map<String, dynamic> params = <String, dynamic>{
      "eventName": eventName
    };
    _channel.invokeMethod('logCustomEvent', params);
  }

  /// Logs a custom event to Braze
  void logCustomEventWithProperties(
      String eventName, Map<String, dynamic> properties) {
    final Map<String, dynamic> params = <String, dynamic>{
      "eventName": eventName,
      "properties": properties
    };
    _channel.invokeMethod('logCustomEvent', params);
  }

  /// Logs a purchase event to Braze
  void logPurchase(
      String productId, String currencyCode, double price, int quantity) {
    final Map<String, dynamic> params = <String, dynamic>{
      "productId": productId,
      "currencyCode": currencyCode,
      "price": price,
      "quantity": quantity
    };
    _channel.invokeMethod('logPurchase', params);
  }

  /// Logs a purchase event to Braze
  void logPurchaseWithProperties(String productId, String currencyCode,
      double price, int quantity, Map<String, dynamic> properties) {
    final Map<String, dynamic> params = <String, dynamic>{
      "productId": productId,
      "currencyCode": currencyCode,
      "price": price,
      "quantity": quantity,
      "properties": properties
    };
    _channel.invokeMethod('logPurchase', params);
  }

  /// Adds an element to a custom attribute array
  void addToCustomAttributeArray(String key, String value) {
    final Map<String, dynamic> params = <String, dynamic>{
      'key': key,
      'value': value
    };
    _channel.invokeMethod('addToCustomAttributeArray', params);
  }

  /// Removes an element from a custom attribute array
  void removeFromCustomAttributeArray(String key, String value) {
    final Map<String, dynamic> params = <String, dynamic>{
      'key': key,
      'value': value
    };
    _channel.invokeMethod('removeFromCustomAttributeArray', params);
  }

  /// Sets a string typed custom attribute
  void setStringCustomUserAttribute(String key, String value) {
    final Map<String, dynamic> params = <String, dynamic>{
      'key': key,
      'value': value
    };
    _channel.invokeMethod('setStringCustomUserAttribute', params);
  }

  /// Sets a double typed custom attribute
  void setDoubleCustomUserAttribute(String key, double value) {
    final Map<String, dynamic> params = <String, dynamic>{
      'key': key,
      'value': value
    };
    _channel.invokeMethod('setDoubleCustomUserAttribute', params);
  }

  /// Sets a boolean typed custom attribute
  void setBoolCustomUserAttribute(String key, bool value) {
    final Map<String, dynamic> params = <String, dynamic>{
      'key': key,
      'value': value
    };
    _channel.invokeMethod('setBoolCustomUserAttribute', params);
  }

  /// Sets a integer typed custom attribute
  void setIntCustomUserAttribute(String key, int value) {
    final Map<String, dynamic> params = <String, dynamic>{
      'key': key,
      'value': value
    };
    _channel.invokeMethod('setIntCustomUserAttribute', params);
  }

  /// Increments an integer typed custom attribute
  void incrementCustomUserAttribute(String key, int value) {
    final Map<String, dynamic> params = <String, dynamic>{
      'key': key,
      'value': value
    };
    _channel.invokeMethod('incrementCustomUserAttribute', params);
  }

  /// Sets a location custom attribute
  void setLocationCustomAttribute(String key, double lat, double long) {
    final Map<String, dynamic> params = <String, dynamic>{
      'key': key,
      'lat': lat,
      'long': long
    };
    _channel.invokeMethod('setLocationCustomAttribute', params);
  }

  /// Sets a date custom attribute
  void setDateCustomUserAttribute(String key, DateTime value) {
    final Map<String, dynamic> params = <String, dynamic>{
      'key': key,
      'value': value.millisecondsSinceEpoch ~/ 1000
    };
    _channel.invokeMethod('setDateCustomUserAttribute', params);
  }

  /// Unsets a custom attribute
  void unsetCustomUserAttribute(String key) {
    _callStringMethod('unsetCustomUserAttribute', 'key', key);
  }

  /// Sets the first name default user attribute
  void setFirstName(String firstName) {
    _callStringMethod('setFirstName', 'firstName', firstName);
  }

  /// Sets the last name default user attribute
  void setLastName(String lastName) {
    _callStringMethod('setLastName', 'lastName', lastName);
  }

  /// Sets the email default user attribute
  void setEmail(String email) {
    _callStringMethod('setEmail', 'email', email);
  }

  /// Sets the dob default user attribute
  void setDateOfBirth(int year, int month, int day) {
    final Map<String, dynamic> params = <String, dynamic>{
      'year': year,
      'month': month,
      'day': day
    };
    _channel.invokeMethod('setDateOfBirth', params);
  }

  /// Sets the gender default user attribute
  void setGender(String gender) {
    _callStringMethod('setGender', 'gender', gender);
  }

  /// Sets the language default user attribute
  void setLanguage(String language) {
    _callStringMethod('setLanguage', 'language', language);
  }

  /// Sets the country default user attribute
  void setCountry(String country) {
    _callStringMethod('setCountry', 'country', country);
  }

  /// Sets the home city default user attribute
  void setHomeCity(String homeCity) {
    _callStringMethod('setHomeCity', 'homeCity', homeCity);
  }

  /// Sets the phone number default user attribute
  void setPhoneNumber(String phoneNumber) {
    _callStringMethod('setPhoneNumber', 'phoneNumber', phoneNumber);
  }

  /// Sets the avatar image url default user attribute
  void setAvatarImageUrl(String avatarImageUrl) {
    _callStringMethod('setAvatarImageUrl', 'avatarImageUrl', avatarImageUrl);
  }

  /// Sets attribution data
  void setAttributionData(
      String network, String campaign, String adGroup, String creative) {
    final Map<String, dynamic> params = <String, dynamic>{
      'network': network,
      'campaign': campaign,
      'adGroup': adGroup,
      'creative': creative
    };
    _channel.invokeMethod('setAttributionData', params);
  }

  /// Registers a push token for the current device with Braze
  void registerPushToken(String pushToken) {
    _callStringMethod('registerPushToken', 'pushToken', pushToken);
  }

  /// Requests an immediate data flush
  void requestImmediateDataFlush() {
    _channel.invokeMethod('requestImmediateDataFlush');
  }

  /// Wipes Data on board the SDK. Please consult iOS and Android-specific
  /// implementation details before using.
  void wipeData() {
    _channel.invokeMethod('wipeData');
  }

  /// Enables the SDK. Please consult iOS and Android-specific
  /// implementation details before using.
  void enableSDK() {
    _channel.invokeMethod('enableSDK');
  }

  /// Disables the SDK. Please consult iOS and Android-specific
  /// implementation details before using.
  void disableSDK() {
    _channel.invokeMethod('disableSDK');
  }

  /// Sets push subscription state for the current user
  void setPushNotificationSubscriptionType(SubscriptionType type) {
    final Map<String, dynamic> params = <String, dynamic>{
      'type': type.toString()
    };
    _channel.invokeMethod('setPushNotificationSubscriptionType', params);
  }

  /// Sets email subscription state for the current user
  void setEmailNotificationSubscriptionType(SubscriptionType type) {
    final Map<String, dynamic> params = <String, dynamic>{
      'type': type.toString()
    };
    _channel.invokeMethod('setEmailNotificationSubscriptionType', params);
  }

  void _callStringMethod(String methodName, String paramName, String value) {
    final Map<String, dynamic> params = <String, dynamic>{paramName: value};
    _channel.invokeMethod(methodName, params);
  }
}

enum SubscriptionType { subscribed, unsubscribed, opted_in }
